var fs = require('fs'),
    sys = require('sys'),
    glob = require('glob'),
    path = require('path'),
    jshint = require('./jshint.js'),
    _cache = {
        directories: {}
    };

function _lint(file, results, config, data) {
    var buffer,
        lintdata;

    try {
        buffer = fs.readFileSync(file, 'utf-8');
    } catch (e) {
        sys.puts("Error: Cant open: " + file);
        sys.puts(e + '\n');
    }

    doLint(buffer, file, results, config, data);
}

function doLint(buffer, file, results, config, data) {
    if (!jshint.JSHINT(buffer, config)) {
        jshint.JSHINT.errors.forEach(function (error) {
            if (error) {
                results.push({file: file, error: error});
            }
        });
    }

    lintdata = jshint.JSHINT.data();

    if (lintdata) {
        lintdata.file = file;
        data.push(lintdata);
    }
}

function isDirectory(aPath) {
    var isDir;

    try {
        if (_cache.directories.hasOwnProperty(aPath)) {
            isDir = _cache.directories[aPath];
        } else {
            isDir = fs.statSync(aPath).isDirectory();
            _cache.directories[aPath] = isDir;
        }
    } catch (e) {
        isDir = false;
    }

    return isDir;
}


function _shouldIgnore(somePath, ignore) {
    function isIgnored(p) {
        var fnmatch = glob.fnmatch(p, somePath, ~(glob.FNM_PATHNAME | glob.FNM_CASEFOLD)),
            lsmatch = isDirectory(p) && p.match(/^[^\/]*\/?$/) &&
                somePath.match(new RegExp("^" + p + ".*"));

        return !!(fnmatch || lsmatch);
    }

    return ignore.some(function (ignorePath) {
        return isIgnored(ignorePath);
    });
}

function _collect(filePath, files, ignore) {
    if (ignore && _shouldIgnore(filePath, ignore)) {
        return;
    }

    if (fs.statSync(filePath).isDirectory()) {
        fs.readdirSync(filePath).forEach(function (item) {
            _collect(path.join(filePath, item), files, ignore);
        });
    } else if (filePath.match(/\.js$/)) {
        files.push(filePath);
    }
}

module.exports = {
    string: function (str, config, reporter) {
        var results = [],
            data = [];

        doLint(str, "Anonymous", results, config, data);

        reporter(results, data, config);
    },

    files: function (targets, config, reporter, ignore) {
        var files = [],
            results = [],
            data = [];

        targets.forEach(function (target) {
            _collect(target, files, ignore);
        });

        files.forEach(function (file) {
            _lint(file, results, config, data);
        });

        _cache = {
            directories: {}
        };

        reporter(results, data, config);
    }
};
